/*
 * ... | pipetype interactive_command [args]
 *
 * Run an interactive command
 *
 * Author: Stewart A. Levin	Sept 13, 1997	  Stanford Exploration Project
 *		Adapted startup.c
 */
#include <sys/types.h>
#if HAVE_TERMIOS_H
#include <termios.h>
#endif
#if HAVE_UNISTD_H
#include <unistd.h>
#endif
#include <sys/stat.h>
#if HAVE_FCNTL_H
#include <fcntl.h>
#endif
#if HAVE_SYS_IOCTL_H
#include <sys/ioctl.h>
#endif
#include <stdio.h>
main(argc,argv,envp)
int argc;
char **argv, **envp;
{
	FILE *fd; 
	int pid;
	int dev_tty;
	char **oargv;
#ifndef O_RDWR
#define O_RDWR 2
#endif
	oargv = argv; /* save for use in printout */
        while(argc>1 && argv[1][0] == '-') {
	    switch(argv[1][1]) {
	    default:
		fprintf(stderr,"unrecognized option -%c",argv[1][1]);
	    }
	    argc--; argv ++;
	}
	if(argc < 2) {
		fprintf(stderr,"usage: %s  command [args]\n", oargv[0]);
		exit(1);
	}
	dev_tty = open("/dev/tty",O_RDWR);
	if(dev_tty < 0) {
		perror(oargv[0]);
		fprintf(stderr,"%s error: unable to open /dev/tty\n",oargv[0]);
		exit(1);
	}

	/*
	 * fork copy of this program to feed script into tty descriptor
	 * via ioctl.  Other copy exec's command with rest of arguments.
	 * Presumably the exec'd command commences to read the shared tty,
	 * assiduously sucking the characters out of the tty that are stuffed
	 * in via ioctl. Echoing of script controlled by current stty settings.
	 * "Type-ahead" before script is input is inadvisable as these may
	 * be intermixed with the scripted commands.
	 *
	 * Note use of fork(), not vfork() to avoid deadlock from sharing
	 * same context.  Also I chose to use child to feed script to avoid
	 * unnecessary orphaning of the command.  The one side effect is that
	 * the child is not 'wait'ed for and will appear as <exiting> under
	 * 'ps' until the exec'ed parent exits and orphans the child. This
	 * can be avoided by feeding the script via an orphaned grandchild.
	 *
	 * Of course this all could be done with pseudo-tty's under 4.[23]
	 * Berkeley Unix.  This would permit "locking the keyboard" while
	 * feeding the script.
	 */

	if(0 == (pid=fork())) { /* child feeds script into stdin */
		int c, i=0;
		char ch;
		long nleft;

		while(EOF != (c = getchar())) {
			ch = c; /* Because first char of c is ch only on VAX */
#ifdef TIOCSTI
			if(-1 == ioctl(dev_tty,TIOCSTI,&ch))
#else
#ifdef TIOCINQ
			if(-1 == ioctl(dev_tty,TIOCINQ,&ch))
#else
#endif/*TIOCINQ*/
			fprintf(stderr,"No ioctl for simulated terminal input ... Exiting\n");
			exit(2);
#endif/*TIOCSTI*/
			{
				perror(oargv[0]);
				fprintf(stderr,"%s: ioctl script input error\n",
					oargv[0]);
				fflush(stderr);
				break;
			}
			i++;
			if((i>128) || (c == '\n')) { /* make editors happy */
				nleft = i;
				while(nleft > 0) 
				    if(-1 == ioctl(dev_tty,FIONREAD,&nleft))nleft=0;
				i = 0;
			}
		}
		_exit(0);
	} 
	else { /* parent execl's command */
		(void) dup2(dev_tty,fileno(stdin));
		(void) close(dev_tty);
#if HAVE_FCNTL_H
		if(0 > fcntl(fileno(stdin),F_SETFD,0)) {
		perror(oargv[0]);
		fprintf(stderr,"%s warning: error changing close-on-exec flag for /dev/tty\n", oargv[0]);
	}
#endif
		if(-1 == execvp(argv[1],argv+1)) {
			perror(oargv[0]);
			fprintf(stderr,"%s: unable to run command %s\n",
				oargv[0], argv[1]);
		}
	}
}
